#!/usr/bin/bash

#usage:
#  dinstall.sh

echo "READ ME FIRST; THIS SCRIPT MAY NOT MATCH YOUR SITUATION"
echo ""
echo "This script assumes it will be running on a freshly-installed CentOS 7 host, with everything at its default setting, which has a functioning Internet connection."
echo ""
echo "It also assumes that you want to set up a diaspora* node with the following attributes:"
echo ""
echo "  a) an Apache and MariaDB stack"
echo "  b) a LetsEncrypt SSl certificate"
echo "  c) an IP-addressable, non-authenticated mail server"
echo "  d) SELinux in permissive mode"
echo "  e) post-install registration at the-federation.info"
echo ""
echo -n "Does this match your situation (y/n): "
read JUNK

if [ "y" != "$JUNK" ]
then
  echo ""
  echo "OK; sorry I can't help.  Please see the installation instructions at https://wiki.diasporafoundation.org/Installation instead."
  exit 1
fi

echo ""
echo "installing diaspora*"
echo ""
echo "Here are a few things you will need to have ready:"
echo ""
echo "  a)  a DNS entry for whatever you intend the pod's hostname to be"
echo "  b)  inbound access authorized on any external firewalls for ports 80 and 443 to this server"
echo "  c)  a functioning email address you will use for administrative contact"
echo "  d)  an SMTP mail server which will accept mail to relay from this server"
echo ""
echo "Hit crtrl-c NOW if you haven't got those ready yet."
echo ""
echo "This script IS NOT completely automated; it does call some interactive commands, so please keep an eye on this terminal as it runs."

#see: https://www.ostricher.com/2014/10/the-right-way-to-get-the-directory-of-a-bash-script/
SOURCE="${BASH_SOURCE[0]}"
while [ -h "$SOURCE" ]; do
  DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
  SOURCE="$( readlink "$SOURCE" )"
  [[ $SOURCE != /* ]] && SOURCE="$DIR/$SOURCE"
done
DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"

echo ""
echo -n "What is the fully-qualified domain name of the diaspora* pod this system will host? (example: podling.oksocial.net) "
read PODURL
echo ""

echo ""
echo -n "What is the human-readable name of the diaspora* pod this system will host? (example: OKSocial: Podling) "
read PODNAME
echo ""

echo ""
echo -n "What is the administrative contact email addres? (example: podmin@oksocial.net) "
read ADMINEMAIL
echo ""

echo ""
echo -n "What is the fully-qualified domain name of the SMTP server this pod will use for sending mail (example: mail.geekwerx.net) "
read MAILSERVER
echo ""

MARIAPASSWORD=`openssl rand -hex 8`

echo ""
echo "Config:"
echo "  PODURL: ${PODURL}"
echo "  PODNAME: ${PODNAME}"
echo "  ADMINEMAIL: ${ADMINEMAIL}"
echo "  MAILSERVER: ${MAILSERVER}"
echo "  MARIAPASSWORD: ${MARIAPASSWORD}"
echo ""
echo "This script IS NOT safe to run again after this point, since that would cause duplicate entries in some files, so please make sure you're totally ready and if you're running on a VM maybe take a snapshot before proceeding."
echo ""
echo -n "Confirm (y/n): "
read JUNK

if [ "y" != "$JUNK" ]
then
  echo ""
  echo "OK; please re-run the script to try again."
  exit 1
fi

echo ""
echo "OK; let's do this thing."

yum -y install epel-release

yum -y install psmisc perl wget mlocate nano screen httpd mod_ssl certbot python2-certbot-apache
#OTHER pre-reqs that _should_ be automatically pulled in by the above:
#  mod_rewrite mod_headers mod_proxy mod_proxy_http mod_proxy_balancer

cp -p $DIR/files/httpd_conf_initial.txt /etc/httpd/conf.d/${PODURL}.conf
chown root:root /etc/httpd/conf.d/${PODURL}.conf
perl -pi -e "s#PODURL#${PODURL}#g" /etc/httpd/conf.d/${PODURL}.conf

mkdir /var/www/${PODURL}
mkdir /var/www/${PODURL}/html
mkdir /var/www/${PODURL}/cgi-bin

systemctl enable httpd
systemctl start httpd

iptables -I INPUT -p tcp -i ens192 --dport 80 -j ACCEPT
iptables -I INPUT -p tcp -i ens192 --dport 443 -j ACCEPT
echo "" >> /etc/rc.local
echo "iptables -I INPUT -p tcp -i ens192 --dport 80 -j ACCEPT" >> /etc/rc.local
echo "iptables -I INPUT -p tcp -i ens192 --dport 443 -j ACCEPT" >> /etc/rc.local

certbot --apache -d ${PODURL}
echo "15 1 * * * /usr/bin/certbot renew" >> /var/spool/cron/root
crontab /var/spool/cron/root

rm -f /etc/httpd/conf.d/${PODURL}-le-ssl.conf
cp -p $DIR/files/httpd_conf_final.txt /etc/httpd/conf.d/${PODURL}.conf
PODURL_ESC=`echo "${PODURL}" | sed 's#\.#\\.#g'`
perl -pi -e "s#PODURL_ESC#${PODURL_ESC}#g" /etc/httpd/conf.d/${PODURL}.conf
perl -pi -e "s#PODURL#${PODURL}#g" /etc/httpd/conf.d/${PODURL}.conf
chown root:root /etc/httpd/conf.d/${PODURL}.conf

service httpd restart

#- Diaspora install
#- see https://wiki.diasporafoundation.org/Installation/CentOS/7?db=mariadb&mode=production#Versions_of_this_guide:

yum -y install tar make automake gcc gcc-c++ git net-tools cmake libcurl-devel libxml2-devel libffi-devel libxslt-devel wget redis ImageMagick nodejs mariadb-devel patch bison bzip2 libtool patch readline-devel ruby sqlite-devel mariadb-server

systemctl enable redis
systemctl start redis

systemctl enable mariadb
systemctl start mariadb

echo "securing the MariaDB installation; PLEASE SET the MariaDB root password to ${MARIAPASSWORD}"
echo "(you can look that password up in ~diaspora/diaspora/config/database.yml later if you need it)"
mysql_secure_installation

adduser diaspora
chmod 755 /home/diaspora

mkdir /var/www/${PODURL}/data
mkdir /var/www/${PODURL}/data/socket
chown diaspora:diaspora /var/www/${PODURL}/data/socket

setenforce permissive
perl -pi -e "s#SELINUX=enforcing#SELINUX=permissive#" /etc/selinux/config

cp -p $DIR/files/dinstall_user.sh ~diaspora/dinstall_user.sh
cp -pR $DIR/files ~diaspora/
chown -R diaspora:diaspora ~diaspora/dinstall_user.sh ~diaspora/files
su -c "~diaspora/dinstall_user.sh '${PODURL}' '${PODNAME}' '${MAILSERVER}' '${ADMINEMAIL}' '${MARIAPASSWORD}'" - diaspora 
rm -rf ~diaspora/dinstall_user.sh ~diaspora/files

cp -p $DIR/files/rootserver.sh ~root/runserver.sh
chown root:root ~root/runserver.sh

cp -p $DIR/files/runserver.sh ~diaspora/runserver.sh
chown diaspora:diaspora ~diaspora/runserver.sh

cp -p $DIR/files/backupserver.sh ~diaspora/backupserver.sh
chown diaspora:diaspora ~diaspora/backupserver.sh

echo "30 1 * * * /home/diaspora/backupserver.sh" >> /var/spool/cron/diaspora
crontab -u diaspora /var/spool/cron/diaspora

screen -d -m ~root/runserver.sh

echo "" >> /etc/rc.local
echo "screen -d -m ~root/runserver.sh" >> /etc/rc.local

wget https://the-federation.info/register/${PODURL}
rm -f ${PODURL}

echo ""
echo "All done; welcome to Diaspora*."
echo ""
echo "Here are a few things you still need to do:"
echo ""
echo "  a)  create the podmin account on your new pod at https://${PODURL}"
echo "  b)  make that account an administrator and moderator"
echo "  c)  arrange remote backups for your new pod (backup files will be in ~diaspora/backups)"
echo "  d)  inform your users that you're ready for them to make accounts"
echo ""
echo "Here are a few things you may wish to do:"
echo ""
echo "  a)  set up PayPal or other donation entries for your new pod"
echo "  b)  set up Twitter or other service integrations for your new pod"
echo "  c)  create a contact on your podmin account for the Diaspora* HQ account at hq@pod.diaspora.software"
echo "  d)  create a contact on your podmin account for the OKSocial HQ account at oksocial@podling.oksocial.net"
echo "  e)  create a contact on your podmin account for the Active POds Log account at active@diapod.net"
echo ""
echo "Regardless, please take a few minutes to look through the config file at ~diaspora/diaspora/config/diaspora.yml before you move on."
echo ""
echo "A good place to start for information on how to do all that is <https://wiki.diasporafoundation.org/FAQ_for_pod_maintainers>."