#!/usr/bin/bash

#usage:
#  dinstall.sh [PODURL] [PODNAME] [MAILSERVER] [ADMINEMAIL] [MARIAPASSWORD]

#see: https://www.ostricher.com/2014/10/the-right-way-to-get-the-directory-of-a-bash-script/
SOURCE="${BASH_SOURCE[0]}"
while [ -h "$SOURCE" ]; do
  DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
  SOURCE="$( readlink "$SOURCE" )"
  [[ $SOURCE != /* ]] && SOURCE="$DIR/$SOURCE"
done
DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"

if [ "" == "$1" ]
then
  echo "ERROR: no pod DNS name; you must provide the fully-qualified domain name of the diaspora* pod this system will host."
  echo ""
  echo "  dinstall.sh [PODURL] [PODNAME] [MAILSERVER] [ADMINEMAIL] [MARIAPASSWORD]"
  
  exit 1
fi

PODURL=$1

if [ "" == "$2" ]
then
  echo "ERROR: no pod name; you must provide the human-readable name of the diaspora* pod this system will host (may not include quote marks)."
  echo ""
  echo "  dinstall.sh [PODURL] [PODNAME] [MAILSERVER] [ADMINEMAIL] [MARIAPASSWORD]"
  
  exit 1
fi

PODNAME=$2

if [ "" == "$3" ]
then
  echo "ERROR: no mail server; you must provide the fully-qualified domain name of the SMTP server this pod will use for sending mail."
  echo ""
  echo "  dinstall.sh [PODURL] [PODNAME] [MAILSERVER] [ADMINEMAIL] [MARIAPASSWORD]"
  
  exit 1
fi

MAILSERVER=$3

if [ "" == "$4" ]
then
  echo "ERROR: no admin email; you must provide an administrative contact email address."
  echo ""
  echo "  dinstall.sh [PODURL] [PODNAME] [MAILSERVER] [ADMINEMAIL] [MARIAPASSWORD]"
  
  exit 1
fi

ADMINEMAIL=$4

if [ "" == "$5" ]
then
  echo "ERROR: no MariaDB password; you must provide a poassword for the root account on the MariaDB server."
  echo ""
  echo "  dinstall.sh [PODURL] [PODNAME] [MAILSERVER] [ADMINEMAIL] [MARIAPASSWORD]"
  
  exit 1
fi

MARIAPASSWORD=$5

HOSTNAME=`hostname`

cd ~

gpg2 --keyserver hkp://pool.sks-keyservers.net --recv-keys 409B6B1796C275462A1703113804BB82D39DC0E3 7D2BAF1CF37B13E2069D6956105BD0E739499BDB

curl -L https://s.diaspora.software/1t | bash

echo '' >> ~/.bashrc
echo '[[ -s "$HOME/.rvm/scripts/rvm" ]] && source "$HOME/.rvm/scripts/rvm"' >> ~/.bashrc

source ~/.bashrc

rvm autolibs read-fail
rvm install 2.4

cd ~
git clone -b master https://github.com/diaspora/diaspora.git
cd diaspora

cp -p $DIR/files/database.txt config/database.yml
perl -pi -e "s#MARIAPASSWORD#${MARIAPASSWORD}#g" config/database.yml
cp -p $DIR/files/diaspora.txt config/diaspora.yml
perl -pi -e "s#PODURL#${PODURL}#g" config/diaspora.yml
perl -pi -e "s#PODNAME#${PODNAME}#g" config/diaspora.yml
perl -pi -e "s#MAILSERVER#${MAILSERVER}#g" config/diaspora.yml
perl -pi -e "s#ADMINEMAIL#${ADMINEMAIL}#g" config/diaspora.yml
perl -pi -e "s#HOSTNAME#${HOSTNAME}#g" config/diaspora.yml

gem install bundler
script/configure_bundler
bin/bundle install --full-index

RAILS_ENV=production bundle exec rake db:create db:migrate
RAILS_ENV=production bin/rake assets:precompile